package data.scripts.world.systems;

import com.fs.starfarer.api.Global;
//import com.fs.starfarer.api.InteractionDialogImageVisual;
import com.fs.starfarer.api.campaign.JumpPointAPI;
import com.fs.starfarer.api.campaign.LocationAPI;
import com.fs.starfarer.api.campaign.OrbitAPI;
import com.fs.starfarer.api.campaign.PlanetAPI;
import com.fs.starfarer.api.campaign.SectorAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.StarSystemAPI;
import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.fleet.FleetMemberType;
import java.awt.Color;
import com.fs.starfarer.api.impl.campaign.ids.Conditions;
import com.fs.starfarer.api.impl.campaign.ids.Submarkets;
import java.util.ArrayList;
import java.util.Arrays;

public class Gemstone {
    
    //addPlanet(String id,  SectorEntityToken focus,name, type, angle, radius, orbitRadius, orbitDays
    
    public void generate(SectorAPI sector) {
        
        StarSystemAPI system = sector.createStarSystem("Gemstone");
        system.getLocation().set(9000, 3200);
        LocationAPI hyper = Global.getSector().getHyperspace();
        
        system.setBackgroundTextureFilename("graphics/AIWar/backgrounds/gemstone.jpg");
        
        float gemstoneRadius = 400f;
        PlanetAPI gemstone = system.initStar("aiw_gemstone", "star_orange", gemstoneRadius);

        float rubyRadius = 100;
        float rubyOrbitRadius = 1600;
        float rubyOrbitPeriod = SystemGenUtils.getOrbitalPeriod(gemstoneRadius, rubyOrbitRadius, 1);
        PlanetAPI ruby = system.addPlanet("aiw_ruby", gemstone, "Ruby", "lava", (float)Math.random()*360, rubyRadius, rubyOrbitRadius, rubyOrbitPeriod);
        ruby.setCustomDescriptionId("planet_ruby");
        
        float emeraldRadius = 180;
        float emeraldOrbitRadius = 3400;
        float emeraldOrbitPeriod = SystemGenUtils.getOrbitalPeriod(gemstoneRadius, emeraldOrbitRadius, 1);
        PlanetAPI emerald = system.addPlanet("aiw_emerald", gemstone, "Emerald", "jungle", (float)Math.random()*360, emeraldRadius, emeraldOrbitRadius, emeraldOrbitPeriod);
        emerald.setCustomDescriptionId("planet_emerald");
        //emerald.setInteractionImage("illustrations", "vacuum_colony");
        
        float minaretOrbitRadius = emeraldRadius + 100;
        float minaretOrbitPeriod = SystemGenUtils.getOrbitalPeriod(emeraldRadius, minaretOrbitRadius, 2);
        SectorEntityToken station_minaret = system.addCustomEntity("aiw_minaret", "Minaret Station", "aiw_station_spirehab", "spire");
        station_minaret.setCircularOrbitPointingDown(emerald, 120, minaretOrbitRadius, minaretOrbitPeriod);
        station_minaret.setInteractionImage("illustrations", "jangala_station");
        station_minaret.setCustomDescriptionId("station_minaret");
        
        SystemGenUtils.addMarketplace("spire",  
                emerald, 
                new ArrayList<>(Arrays.asList((SectorEntityToken) emerald, station_minaret)),  
                "Emerald",  
                5,  
                new ArrayList<>(Arrays.asList(Conditions.JUNGLE, Conditions.POPULATION_5, "aiw_inorganic_populace",
                        Conditions.ORGANICS_COMPLEX, Conditions.ORGANICS_COMPLEX, Conditions.ORGANICS_COMPLEX, Conditions.LIGHT_INDUSTRIAL_COMPLEX,
                        Conditions.TRADE_CENTER, Conditions.ORBITAL_STATION)),  
                new ArrayList<>(Arrays.asList(Submarkets.SUBMARKET_STORAGE, Submarkets.SUBMARKET_BLACK, Submarkets.SUBMARKET_OPEN)),  
                0.3f
        );
        
        float diamondRadius = 165;
        float diamondOrbitRadius = 5100;
        float diamondOrbitPeriod = SystemGenUtils.getOrbitalPeriod(gemstoneRadius, diamondOrbitRadius, 1);
        
        float zirconRadius = 35;
        float zirconOrbitRadius = diamondRadius + 200;
        float zirconOrbitPeriod = SystemGenUtils.getOrbitalPeriod(diamondRadius, zirconOrbitRadius, 2);
        
        PlanetAPI diamond = system.addPlanet("aiw_diamond", gemstone, "Diamond", "rocky_ice", (float)Math.random()*360, diamondRadius, diamondOrbitRadius, diamondOrbitPeriod);
        diamond.setFaction("spire");
        diamond.setInteractionImage("illustrations", "city_from_above");
        diamond.setCustomDescriptionId("planet_diamond");
        diamond.getSpec().setGlowTexture(Global.getSettings().getSpriteName("hab_glows", "volturn"));
        diamond.getSpec().setGlowColor(new Color(255,255,255,255));
        diamond.getSpec().setUseReverseLightForGlow(true);
        diamond.applySpecChanges();
        
        PlanetAPI zircon = system.addPlanet("aiw_zircon", diamond, "Zircon", "barren", (float)Math.random()*360, zirconRadius, zirconOrbitRadius, zirconOrbitPeriod);
        zircon.setFaction("spire");
        zircon.setCustomDescriptionId("planet_zircon");
        
        SystemGenUtils.addMarketplace("spire",  
                diamond,  
                new ArrayList<>(Arrays.asList((SectorEntityToken)diamond, (SectorEntityToken)zircon)),  
                "Diamond",  
                6,  
                new ArrayList<>(Arrays.asList(Conditions.ICE, Conditions.POPULATION_6, Conditions.MILITARY_BASE, Conditions.SPACEPORT, "aiw_inorganic_populace",
                        Conditions.ORE_COMPLEX, Conditions.ORE_COMPLEX, Conditions.ORE_COMPLEX, Conditions.ORE_COMPLEX, 
                        Conditions.ORE_REFINING_COMPLEX, Conditions.ORE_REFINING_COMPLEX, Conditions.AUTOFAC_HEAVY_INDUSTRY,
                        Conditions.ANTIMATTER_FUEL_PRODUCTION, Conditions.HEADQUARTERS, Conditions.URBANIZED_POLITY, Conditions.VOLATILES_DEPOT)),  
                new ArrayList<>(Arrays.asList(Submarkets.SUBMARKET_STORAGE, Submarkets.SUBMARKET_BLACK, Submarkets.SUBMARKET_OPEN)),  
                0.3f
        );
        
        float relayOrbitRadius = 700;
        float relayOrbitPeriod = SystemGenUtils.getOrbitalPeriod(diamondRadius, relayOrbitRadius, 2);
        SectorEntityToken relay = system.addCustomEntity("aiw_gemstone_relay", // unique id
        "Gemstone Relay", // name - if null, defaultName from custom_entities.json will be used
        "comm_relay", // type of object, defined in custom_entities.json
        "spire"); // faction
        relay.setCircularOrbit(diamond, 120, relayOrbitRadius, relayOrbitPeriod);
        
        float sapphireRadius = 290;
        float sapphireOrbitRadius = 8500;
        float sapphireOrbitPeriod = SystemGenUtils.getOrbitalPeriod(gemstoneRadius, sapphireOrbitRadius, 1);
        
        float aluminaRadius = 100;
        float aluminaOrbitRadius = sapphireRadius + 600;
        float aluminaOrbitPeriod = SystemGenUtils.getOrbitalPeriod(sapphireRadius, aluminaOrbitRadius, 2);
        
        PlanetAPI sapphire = system.addPlanet("aiw_sapphire", gemstone, "Sapphire", "ice_giant", (float)Math.random()*360, sapphireRadius, sapphireOrbitRadius, sapphireOrbitPeriod);
        PlanetAPI alumina = system.addPlanet("aiw_alumina", sapphire, "Alumina", "cryovolcanic", (float)Math.random()*360, aluminaRadius, aluminaOrbitRadius, aluminaOrbitPeriod);
        alumina.setCustomDescriptionId("planet_alumina");
        alumina.setInteractionImage("illustrations", "vacuum_colony");
        alumina.setFaction("spire");
                
        SystemGenUtils.addMarketplace("spire",  
                alumina,  
                new ArrayList<>(Arrays.asList((SectorEntityToken) alumina)),  
                "Alumina",  
                4,  
                new ArrayList<>(Arrays.asList(Conditions.ICE, Conditions.POPULATION_4, Conditions.OUTPOST, "aiw_inorganic_populace", 
                        Conditions.VOLATILES_COMPLEX, Conditions.VOLATILES_COMPLEX, Conditions.ORE_COMPLEX, Conditions.STEALTH_MINEFIELDS)),  
                new ArrayList<>(Arrays.asList(Submarkets.SUBMARKET_STORAGE, Submarkets.SUBMARKET_BLACK, Submarkets.SUBMARKET_OPEN)),  
                0.3f  
        );
        
        FleetMemberAPI member = Global.getFactory().createFleetMember(FleetMemberType.SHIP, "spire_champion_Hull");
        member.getRepairTracker().setMothballed(true);
        alumina.getMarket().getSubmarket(Submarkets.SUBMARKET_STORAGE).getCargo().getMothballedShips().addFleetMember(member);
                
        system.addAsteroidBelt(gemstone, 125, 6400, 500, 500, 720);
        
        JumpPointAPI jumpPoint = Global.getFactory().createJumpPoint("aiw_gemstone_jump", "Amethyst Bridge");
        OrbitAPI orbit = Global.getFactory().createCircularOrbit(emerald, 0, 500, SystemGenUtils.getOrbitalPeriod(emeraldRadius, 500, 2));
        jumpPoint.setOrbit(orbit);
        jumpPoint.setRelatedPlanet(emerald);
        
        jumpPoint.setStandardWormholeToHyperspaceVisual();
        system.addEntity(jumpPoint);

        system.autogenerateHyperspaceJumpPoints(true, true);
    }
}
